﻿using UnityEngine;
using System.Collections;
using UnityEngine.UI;
using System.Collections.Generic;

public class useBLE : MonoBehaviour 
{
	public GameObject deviceBanner;
	public GameObject connectingBanner;
	public GameObject connectedButton;


	public Text connectionStatus;
	public Text mcuIDText;
	public Text batteryText;


	private string[] devices;
	private static bool btInitialized = false; //In case we change scenes
	private bool currentlyScanning = false;
	private List<GameObject> buttonObjects = new List<GameObject>();

	void OnDestroy()
	{
		//We need to unsubscribe the delegates because it has been destroyed.
		NSB_BLE.instance.populateBLE_DeviceStatusUpdateCallBack(getDeviceStatus, true);
		NSB_BLE.instance.populateBLE_BluetoothStatusUpdateCallBack(getBTStatus, true);
		NSB_BLE.instance.populateBLE_BreakConnection(connectionBroken, true);

		NSB_EEG.instance.assignBatteryStatus(null);
		NSB_BLE.instance.assignScanCallBack(null);
		NSB_BLE.instance.startStopScanning(false);
		currentlyScanning = false;

	}

	void Start()
	{
		Debug.Log("USE BLE ACTIVE");
		if(btInitialized == false)
		{
			initializeBT();
			btInitialized = true;
		}
		else //Will be ran if we come back from another scene
		{
			//Reinitialize the delegates
			reinitializeBT();


		}
	}

	void initializeBT()
	{
		NSB_BLE.instance.initializeBT(clearList, getDeviceStatus,getBTStatus);
		NSB_BLE.instance.assignErrorLogDelegate(errorLog);
		NSB_EEG.instance.assignBatteryStatus(printBattery);
		NSB_BLE.instance.assignScanCallBack(populateSingleDeviceList);
		NSB_BLE.instance.startStopScanning(true);
	}

	void reinitializeBT()
	{
		#if (!UNITY_EDITOR)
		NSB_BLE.instance.populateBLE_DeviceStatusUpdateCallBack(getDeviceStatus);
		NSB_BLE.instance.populateBLE_BluetoothStatusUpdateCallBack(getBTStatus);

		NSB_EEG.instance.assignBatteryStatus(printBattery);

		NSB_BLE.instance.assignScanCallBack(populateSingleDeviceList);
		NSB_BLE.instance.startStopScanning(true);
		currentlyScanning = true;


		getDeviceStatus(NSB_BLE.instance.getDeviceConnectionStatus());
		clearList();

		//We are connected to a device
		if(string.Equals(NSB_BLE.instance.getConnectedString(), NSB_BLE.instance.getDeviceConnectionStatus()))
		{
			string connectedDevice = NSB_BLE.instance.getCurrentConnectedDevice();
			connectedButton.SetActive(true);
			connectedButton.GetComponent<bluetoothButton>().deviceID.text = connectedDevice; //connectedDevice.Substring(16, connectedDevice.Length - 16);

			#if UNITY_IOS
			connectedButton.GetComponent<bluetoothButton>().deviceID.fontSize = 24;
			#endif

			connectedButton.GetComponent<bluetoothButton>().actualID = connectedDevice;
			NSB_BLE.instance.populateBLE_BreakConnection(connectionBroken);
		} 
		#endif
	}

	void errorLog(string error)
	{
		Debug.LogError("NSB SDK ERROR LOG: " + error);
	}

	void clearList()
	{
		foreach(GameObject go in buttonObjects)
		{
			GameObject.Destroy(go);
		}

		buttonObjects.Clear();
		NSB_BLE.instance.clearList();

		Invoke("clearList", 4.0f); 
	}

	void populateSingleDeviceList(string address)
	{
		GameObject go = GameObject.Instantiate(deviceBanner);
		go.GetComponent<bluetoothButton>().deviceID.text = address;

		#if UNITY_IOS
		go.GetComponent<bluetoothButton>().deviceID.fontSize = 24;
		#endif
		go.GetComponent<bluetoothButton>().actualID = address;

		go.GetComponent<bluetoothButton>().thisButton.onClick.AddListener( delegate {connectBT(go.GetComponent<bluetoothButton>().actualID); } );
		go.GetComponent<bluetoothButton>().thisButton.onClick.AddListener( delegate {go.SetActive(false); } );
		go.GetComponent<bluetoothButton>().crossButton.GetComponent<Button>().onClick.AddListener( delegate {breakConnection(); } );

		go.transform.SetParent(this.transform);
		go.transform.localScale = new Vector3(1.0f,1.0f,1.0f);
		go.transform.localPosition = new Vector3(0.0f, 0.0f - 110 * buttonObjects.Count, 1.0f);

		buttonObjects.Add(go);
	}

	/*void populateDeviceList(string[] devices)
	{
		Debug.Log("Populate device list is called");
		//Note that this is a very bad way to do it. Its just an example. Ideally, you may want an object manager or some other way to only reuse/destroy buttons that you don't need.
		foreach(GameObject go in buttonObjects)
		{
			GameObject.Destroy(go);
		}

		buttonObjects.Clear();

		if(devices != null)
		{
			for(int i = 0; i < devices.Length; ++i)
			{
				Debug.Log("devices are " + devices[i]);

				GameObject go = GameObject.Instantiate(deviceBanner);
				go.GetComponent<bluetoothButton>().deviceID.text = devices[i].Substring(16, devices[i].Length - 16);
				#if UNITY_IOS
				go.GetComponent<bluetoothButton>().deviceID.fontSize = 24;
				#endif
				go.GetComponent<bluetoothButton>().actualID = devices[i];

				go.GetComponent<bluetoothButton>().thisButton.onClick.AddListener( delegate {connectBT(go.GetComponent<bluetoothButton>().actualID); } );
				go.GetComponent<bluetoothButton>().thisButton.onClick.AddListener( delegate {go.SetActive(false); } );
				go.GetComponent<bluetoothButton>().crossButton.GetComponent<Button>().onClick.AddListener( delegate {breakConnection(); } );

				go.transform.SetParent(this.transform);
				go.transform.localScale = new Vector3(1.0f,1.0f,1.0f);
				go.transform.localPosition = new Vector3(0.0f, 0.0f - 110 * buttonObjects.Count, 1.0f);
				buttonObjects.Add(go);
			}
		}

		//Auto refreshes every 4 seconds
		Invoke("getBLEList", 4.0f);
	}*/

	void getDeviceStatus(string status)
	{
		connectionStatus.text = "Connection Status: " + status;
	}


	void getBTStatus(string status)
	{
		Debug.Log("The current bluetooth status is " + status);
	}

	public void connectionSucceed(string btDetails)
	{
		currentlyScanning = true;
		Debug.Log("BLE Connected Callback successful");
		connectedButton.SetActive(true);
		connectedButton.GetComponent<bluetoothButton>().deviceID.text = btDetails; //btDetails.Substring(16, btDetails.Length - 16);
		#if UNITY_IOS
		connectedButton.GetComponent<bluetoothButton>().deviceID.fontSize = 24;
		#endif

		connectedButton.GetComponent<bluetoothButton>().actualID = btDetails;
	}

	//This is a callBack.
	public void connectionBroken(string device)
	{
		//The connection is already broken if it reaches here.
		Debug.Log("Connection broken " + device);
		connectedButton.SetActive(false);
		connectedButton.GetComponent<bluetoothButton>().deviceID.text = string.Empty;
		connectedButton.GetComponent<bluetoothButton>().actualID = string.Empty;

		mcuIDText.text = "";
	}

	//This is if we want to break the connection
	public void breakConnection()
	{
		Debug.Log("Break connection " + connectedButton.GetComponent<bluetoothButton>().actualID);

		NSB_BLE.instance.disconnectBT(connectedButton.GetComponent<bluetoothButton>().actualID);
		connectedButton.SetActive(false);
	}

	public void getMCUID()
	{
		mcuIDText.text = NSB_BLE.instance.getMCUID(connectedButton.GetComponent<bluetoothButton>().actualID);
	}

	public void startStopScanning()
	{
		if(currentlyScanning == true)
		{
			currentlyScanning = false;
			NSB_BLE.instance.startStopScanning(false);
		}
		else
		{
			currentlyScanning = true;
			NSB_BLE.instance.startStopScanning(true);
		}
	}

	private void connectBT(string device)
	{
		Debug.Log("Connect BT");
		if(NSB_BLE.instance.connectBT(device, connectionSucceed, connectionBroken) == false)
		{
			connectingBanner.SetActive(true);
		}
	}



	private void printBattery(string battery)
	{
		if(batteryText == null) //In case we change scenes
			return;
		batteryText.text = "Battery: " + battery + "%";
	}
}
