﻿using UnityEngine;
using System.Collections;
using UnityEngine.UI;

public class useEEG : MonoBehaviour {

	public Text attentionText; 
	public Text relaxationText;

	public Text EEGStatus;
	public Text GoodConnection;

	public Text accelerometerX;
	public Text accelerometerY;
	public Text accelerometerZ;

	public Text channel1;
	public Text channel2;
	public Text channel3;
	public Text channel4;

	public GameObject[] activateUponConnection; 

	//Pass data to other classes
	//Static so that other classes can access it. 
	//We can also put the delegates that belong to the other classes into the NSB_EEG delegates. This is just another way to send the data over
	public static float[] mentalStateData = new float[4]; //Array to hold Mental State Data. Relaxation, Attention
	private static bool eegInitialized = false; //In case we change scenes

	private bool currentlyReceivingEEG = false;


	#region graphRecording
	private enum graphTypes
	{
		ATTENTION,
		RELAXATION,
	};

	private const int NUM_MENTALSTATES = 4;
	private string writePath = string.Empty;
	private string[] mentalGraphNames = {"Graph_Relaxation", "Graph_Attention"}; //We don't measure stress atm, but we just put it there first

	private GraphRecording[] recordMentalGraph = new GraphRecording[2];

	[HideInInspector] public float[] EEG_MentalStateBufferRecord	= new float[NUM_MENTALSTATES];	// Mental state buffer : relaxation, attention, mental workload

	private delegate void graphRecorder(graphTypes typeOfGraph);
	graphRecorder writeRecordGraph;

	private System.DateTime startingTime;

	#endregion


	// Use this for initialization
	void Start () 
	{
		#if !UNITY_EDITOR
		writePath = Application.persistentDataPath + "/";
		#else
		writePath = Application.dataPath + "/SaveFile/";
		#endif

		if(eegInitialized == false)
		{
			initializeEEG();
			eegInitialized = true;
		}
		else //Will be ran if we come back from another scene
		{
			assignDelegates();

			//If we are connected to another device
			if(string.Equals(NSB_BLE.instance.getConnectedString(), NSB_BLE.instance.getDeviceConnectionStatus()))
			{
				connectFromDevice(NSB_BLE.instance.getCurrentConnectedDevice());
			} 
			else
			{
				NSB_BLE.instance.populateBLE_FinishConnecting(connectFromDevice);
			}
		}
	}

	void OnDestroy()
	{
		NSB_BLE.instance.populateBLE_BreakConnection(disconnectFromDevice, true);
		NSB_BLE.instance.populateBLE_FinishConnecting(connectFromDevice, true);
	}

	void initializeEEG()
	{
		NSB_BLE.instance.populateBLE_FinishConnecting(connectFromDevice);
		assignDelegates();
	}

	void assignDelegates()
	{
		NSB_EEG.instance.assignAttentionDelegate(grabAttention);
		NSB_EEG.instance.assignRelaxationDelegate(grabRelaxation);
		NSB_EEG.instance.assignAccDelegate(grabAccelerometer);
		NSB_EEG.instance.assignChannelDelegate(grabChannelStatus);
		NSB_EEG.instance.assignGoodConnectionCheckDelegate(grabGoodConnection);
	}

	void grabAttention(float attention)
	{
		attentionText.text = "Attention: " + attention.ToString();
		mentalStateData[1] = attention;

		if(writeRecordGraph != null)
			writeRecordGraph(graphTypes.ATTENTION);
	}

	void grabGoodConnection(bool goodConnection)
	{
		Debug.Log("Good connection is " + goodConnection);
	}

	void grabRelaxation(float relaxation)
	{
		Debug.Log("relaxation gotten is " + relaxation);
		relaxationText.text = "Relaxation: " + relaxation.ToString();
		mentalStateData[0] = relaxation;

		if(writeRecordGraph != null)
			writeRecordGraph(graphTypes.RELAXATION);
	}


	void grabAccelerometer(float[] acc)
	{
		accelerometerX.text = "Accelerometer X: " + acc[0].ToString();
		accelerometerY.text = "Accelerometer Y: " + acc[1].ToString();
		accelerometerZ.text = "Accelerometer Z: " + acc[2].ToString();
	}

	void grabChannelStatus(bool[] chnStatus)
	{
		channel1.text = "Channel 1: " + chnStatus[0].ToString();
		channel2.text = "Channel 2: " + chnStatus[1].ToString();
		channel3.text = "Channel 3: " + chnStatus[2].ToString();
		channel4.text = "Channel 4: " + chnStatus[3].ToString();
	}

	#region button usage

	public void EEG_StartReceiving()
	{
		NSB_EEG.instance.EEG_Start();
	}

	public void EEG_StopReceiving()
	{
		NSB_EEG.instance.EEG_Stop();
	}

	public void swapEEG_StartStopReceiving()
	{
		if(currentlyReceivingEEG)
			EEG_StopReceiving();
		else
			EEG_StartReceiving();

		currentlyReceivingEEG = !currentlyReceivingEEG;
	}

	private void disconnectFromDevice(string device)
	{
		currentlyReceivingEEG = false;

		foreach(GameObject go in activateUponConnection)
		{
			go.SetActive(false);

			if(go.GetComponent<GenericTextSwap>())
				go.GetComponent<GenericTextSwap>().resetText();
		}

		//This resets everytime its called. So we need to initialize it again
		NSB_BLE.instance.populateBLE_FinishConnecting(connectFromDevice);
	}

	private void connectFromDevice(string device)
	{
		currentlyReceivingEEG = true;
		EEG_StartReceiving();

		foreach(GameObject go in activateUponConnection)
		{
			go.SetActive(true);
		}

		//This resets everytime its called. So we need to initialize it again
		NSB_BLE.instance.populateBLE_BreakConnection(disconnectFromDevice);
	}

	#endregion

	#region recording

	public void initRecording()
	{
		startingTime = System.DateTime.UtcNow;

		for(int i = 0; i < recordMentalGraph.Length; ++i)
		{
			recordMentalGraph[i] = new GraphRecording();
			recordMentalGraph[i].initFile(writePath, mentalGraphNames[i] + " " +  startingTime.ToString("yyyy-MM-ddTHH-mm-ss"));
		}

		writeRecordGraph = writeToGraphs;
	}

	private void writeToGraphs(graphTypes whichGraph)
	{
		if(whichGraph == graphTypes.RELAXATION)
			writeToRelaxationGraph();
		else if(whichGraph == graphTypes.ATTENTION)
			writeToAttentionGraph();
	}

	private void writeToRelaxationGraph()
	{
		recordMentalGraph[0].writeToFile(mentalStateData[0].ToString());
	}

	private void writeToAttentionGraph()
	{
		recordMentalGraph[1].writeToFile(mentalStateData[1].ToString());
	}

	//Called by the button
	public void stopRecording()
	{
		//If this is null, means initRecording wasn't even called. So no need to close the files. 
		//This can also be null if we exit the graph page without recording
		if(writeRecordGraph == null)
			return;

		writeRecordGraph = null;
		for(int i = 0; i < recordMentalGraph.Length; ++i)
		{
			recordMentalGraph[i].closeFile();
		}
	}

	#endregion

}
