﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class DrawEEGGraphEX_MentalState : MonoBehaviour {
	
	public const int WINDOW_SIZE = 100;
	public const int BUFFER_SIZE = 1;
	public const int FRAME_RATE = 30;
	public const float MAX_VARIANCE = 1f;

	public bool simulateSignal = false;
	public float maxValue = 0f;

	public int mentalStateType = 0;
	public Material myMaterial;
	public Color[] baseColors;
	public float lineWidth = 1;
	public float redLineWidth = 1.5f;
	public float interval = 10;
	public float amplitude = 10;
	public int steps = 5;
	public Text chMean;
	public CanvasGroup nosignalObj;
	public RectTransform highestRange;
	public RectTransform averageRange;
	public RectTransform lowestRange;
	public int rangeInterval = 10;
	public bool showGraph = true;

	private Mesh mesh;
	private float[] eegWindow;
	private float[] rollAvg;
	private int windowStart;
	private int frameCount;
	private int updateCount;
	private bool doUpdate = false;		

	private Vector3 p0;
	private Vector3 p1;
	private Vector3 p2;
	private Vector3 p3;


	// Use this for initialization
	void Awake () 
	{
		MeshFilter mf = (MeshFilter)gameObject.AddComponent<MeshFilter>();
		MeshRenderer mr = (MeshRenderer)gameObject.AddComponent<MeshRenderer>();
		
		mesh = new Mesh();
		mesh.name = "MyMesh";
		mf.mesh = mesh;
		mr.material = myMaterial;
		
		eegWindow = new float[WINDOW_SIZE];
		rollAvg = new float[WINDOW_SIZE];
		windowStart = 0;
		frameCount = 0;
		updateCount = 0;
		//nosignalObj.alpha = 1;
	}
	
	void OnEnable ()
	{		
		StartCoroutine(UpdateWave());
	}

	public void ResetWindowStart ()
	{
		windowStart = 0;
		updateCount = 0;
	}
	
	public void UpdateEEGSignal ( float EEG_value )
	{ 		
		//nosignalObj.alpha = 0;
		if (float.IsNaN(EEG_value))
			EEG_value = 0; 
		eegWindow[windowStart] = EEG_value;
		windowStart++;
		if (windowStart >= WINDOW_SIZE) windowStart = 0;
	}
	
	void CheckRange ()
	{
		float _highest = -1;
		float _lowest = -1;
		float _average = 0;
		int _count = 0;
		int _min = 0;
		if (updateCount - rangeInterval - 1 >= 0)
			_min = updateCount - rangeInterval - 1;
		
		for (int i = updateCount; i >= _min; i--) 
		{
			if (eegWindow[i] > _highest || _highest == -1) _highest = eegWindow[i];
			if (eegWindow[i] < _lowest || _lowest == -1) _lowest = eegWindow[i];
			
			_count ++;
			_average += eegWindow[i];
		} 
		_average = _average / _count;

		float y = GetClampedValue(_highest * amplitude / (maxValue > 0 ? maxValue : 1));
		highestRange.anchoredPosition = new Vector3 (highestRange.anchoredPosition.x, y);

		y = GetClampedValue(_average * amplitude / (maxValue > 0 ? maxValue : 1));
		averageRange.anchoredPosition = new Vector3 (averageRange.anchoredPosition.x, y);

		y = GetClampedValue(_lowest * amplitude / (maxValue > 0 ? maxValue : 1));
		lowestRange.anchoredPosition = new Vector3 (lowestRange.anchoredPosition.x, y);
	}

	IEnumerator UpdateWave ()
	{
		doUpdate = true;
		while (doUpdate)
		{
			if (frameCount % FRAME_RATE == 0) 
			{
				frameCount = 0;
				// Grab buffer
				float value = 0f;
				if(simulateSignal)
				{
					value = Random.Range (0f,maxValue);
				}
				else
				{ 
					value = useEEG.mentalStateData[mentalStateType] ;
				}


				eegWindow[updateCount] = value;


				int index1 = updateCount - 1 < 0 ? WINDOW_SIZE + updateCount - 1 : updateCount - 1;
				//int index2 = updateCount - 2 < 0 ? WINDOW_SIZE + updateCount - 2 : updateCount - 2;
				//int index3 = updateCount - 3 < 0 ? WINDOW_SIZE + updateCount - 3 : updateCount - 3;

				if( updateCount == 0 )	
					rollAvg[updateCount] = eegWindow[0];
				else
					rollAvg[updateCount] = 0.8f*rollAvg[updateCount-1] + 0.2f*eegWindow[index1];
				
				CheckRange();

				windowStart++;
				updateCount++;
				if (windowStart >= WINDOW_SIZE) 
				{ 
					windowStart = 0;
					updateCount = 0;
				}
			}
			UpdateGraph();
			frameCount++;
			yield return null;
		}

	}

	public void ClearGraph ()
	{
		for (int i = 0; i < WINDOW_SIZE; i++)
			eegWindow[i] = 0;		
		for (int i = 0; i < WINDOW_SIZE; i++)
			rollAvg[i] = 0;
	}

	public void ShowGraph (bool state)
	{
		showGraph = state;
		UpdateGraph();
	}

	public void UpdateDisplay ()
	{
		UpdateGraph();
	}

	float GetGraphValue (int index)
	{
		if (index < 0) index += WINDOW_SIZE;
		if (index >= WINDOW_SIZE) index -= WINDOW_SIZE;

		float value = eegWindow[index] * amplitude;
		if (maxValue > 0)
			value /= maxValue;
		return value;
	}

	float GetRollingAvg (int index)
	{
		if (index < 0) index += WINDOW_SIZE;
		if (index >= WINDOW_SIZE) index -= WINDOW_SIZE;

		float value = rollAvg[index] * amplitude;
		if (maxValue > 0)
			value /= maxValue;
		return value;
	}

	float CatmullRomSpline (float t, float v0, float v1, float v2, float v3)
	{
		return 0.5f * ((2 * v1) +
			(v2 - v0) * t +
			(2 * v0 - 5 * v1 + 4 * v2 - v3) * t * t +
			(3 * v1 - v0 - 3 * v2 + v3) * t * t * t);
	}

	float GetClampedValue (float val)
	{
		if (val > MAX_VARIANCE * amplitude)
			val = MAX_VARIANCE * amplitude;
		else if (val < 0)
			val = 0;
		return val;
	}

	void UpdateGraph () 
	{
		if (mesh == null) return;	// Only update when mesh is ready
		if (!showGraph) 
		{
			mesh.Clear ();
			return;
		}

		Vector3[] vertices = new Vector3[((WINDOW_SIZE - 1) * (steps)) * 4 * 2];
		Color[] vertexColors = new Color[vertices.Length];
		int[] tris = new int[((WINDOW_SIZE - 1) * (steps)) * 4 * 3 * 2];
		
		// Build  vertices
		int vcount = 0;
		float xpos = interval * frameCount / (float) FRAME_RATE;
		int cursor = windowStart;

		for (int i = 0; i < WINDOW_SIZE-1; i++) 
		{
			float value1 = i == 0 ? GetGraphValue(cursor) : GetGraphValue(cursor-1);
			float value2 = GetGraphValue(cursor);
			float value3 = GetGraphValue(cursor+1);
			float value4 = i+2 == WINDOW_SIZE ? GetGraphValue(cursor+1) : GetGraphValue(cursor+2);

			p0 = new Vector3((float)i * interval - xpos, value1, 0);
			p1 = new Vector3((float)i * interval + interval - xpos, value2, 0);
			p2 = new Vector3((float)i * interval + 2f * interval - xpos, value3, 0);
			p3 = new Vector3((float)i * interval + 3f * interval - xpos, value4, 0);

			Vector3 segLineStart = new Vector3((float)i * interval + interval - xpos, GetClampedValue(value2), 0);
			Vector3 segLineEnd = p2;
			for (int j = 1; j <= steps; j++)
			{
				float t = j * 1/(float)steps;
				float x = CatmullRomSpline(t, p0.x, p1.x, p2.x, p3.x);
				float y = CatmullRomSpline(t, p0.y, p1.y, p2.y, p3.y);
				segLineEnd = new Vector3(x, GetClampedValue(y), 0);

				Vector3 startPt = new Vector3(segLineEnd.y, segLineStart.x, 0);
				Vector3 endPt = new Vector3(segLineStart.y, segLineEnd.x, 0);
				Vector3 perpendicular = (startPt - endPt).normalized * lineWidth;

				vertices[vcount] = (segLineStart - perpendicular);
				vertices[vcount + 1] = (segLineStart + perpendicular);
				vertices[vcount + 2] = (segLineEnd - perpendicular);
				vertices[vcount + 3] = (segLineEnd + perpendicular);

				vcount += 4;
				segLineStart = segLineEnd;
			}
			cursor++;
			if (cursor >= WINDOW_SIZE)
			{
				cursor = 0;
			}
		}

		cursor = windowStart;
		for (int i = 0; i < WINDOW_SIZE-1; i++) 
		{
			float value1 = i == 0 ? GetRollingAvg(cursor) : GetRollingAvg(cursor-1);
			float value2 = GetRollingAvg(cursor);
			float value3 = GetRollingAvg(cursor+1);
			float value4 = i+2 == WINDOW_SIZE ? GetRollingAvg(cursor+1) : GetRollingAvg(cursor+2);

			p0 = new Vector3((float)i * interval - xpos, value1, 0);
			p1 = new Vector3((float)i * interval + interval - xpos, value2, 0);
			p2 = new Vector3((float)i * interval + 2f * interval - xpos, value3, 0);
			p3 = new Vector3((float)i * interval + 3f * interval - xpos, value4, 0);
			
			Vector3 segLineStart = new Vector3((float)i * interval + interval - xpos, GetClampedValue(value2), 0);
			Vector3 segLineEnd = p2;
			for (int j = 1; j <= steps; j++)
			{
				float t = j * 1/(float)steps;
				float x = CatmullRomSpline(t, p0.x, p1.x, p2.x, p3.x);
				float y = CatmullRomSpline(t, p0.y, p1.y, p2.y, p3.y);
				segLineEnd = new Vector3(x, GetClampedValue(y), 0);

				Vector3 startPt = new Vector3(segLineEnd.y, segLineStart.x, 0);
				Vector3 endPt = new Vector3(segLineStart.y, segLineEnd.x, 0);
				Vector3 perpendicular = (startPt - endPt).normalized * redLineWidth;

				vertices[vcount] = (segLineStart - perpendicular);
				vertices[vcount + 1] = (segLineStart + perpendicular);
				vertices[vcount + 2] = (segLineEnd - perpendicular);
				vertices[vcount + 3] = (segLineEnd + perpendicular);

				vcount += 4;
				segLineStart = segLineEnd;
			}
			cursor++;
			if (cursor >= WINDOW_SIZE)
			{
				cursor = 0;
			}
		}

		for (int i = 0; i < vertices.Length; i++)
		{
			if (i < vertices.Length/2)
				vertexColors[i] = baseColors[0];
			else
				vertexColors[i] = baseColors[1];
		}

		// Triangles
		int tcount = 0;
		for (int i = 0; i < vertices.Length/2 - 2; i += 2)
		{
			tris[tcount * 3] = i;
			tris[tcount * 3 + 1] = i + 1;
			tris[tcount * 3 + 2] = i + 2;
			tcount++;
			tris[tcount * 3] = i + 1;
			tris[tcount * 3 + 1] = i + 3;
			tris[tcount * 3 + 2] = i + 2;
			tcount++;
		}
		for (int i = vertices.Length/2; i < vertices.Length - 2; i += 2)
		{
			tris[tcount * 3] = i;
			tris[tcount * 3 + 1] = i + 1;
			tris[tcount * 3 + 2] = i + 2;
			tcount++;
			tris[tcount * 3] = i + 1;
			tris[tcount * 3 + 1] = i + 3;
			tris[tcount * 3 + 2] = i + 2;
			tcount++;
		}

		mesh.Clear();
		mesh.vertices = vertices;
		mesh.triangles = tris;
		mesh.colors = vertexColors;
		
		mesh.RecalculateNormals();
		mesh.RecalculateBounds();
		;
	}
}

